package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesErrorLineOff;
import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesErrorLineOffDetail;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:06 下午
 * @Description:
 **/
public interface IErrorLineOffService {

    /**
     * 根据工单号查询异常下线集合
     * @param organizeCode 组织代码
     * @param workOrderNo  生产工单号
     * @return 异常下线集合
     */
    @ApiOperation(value = "根据工单号查询异常下线集合", notes = "根据工单号查询异常下线集合")
    List<MesErrorLineOff> getErrorLineOffListByWorkOrderNo(String organizeCode, String workOrderNo);

    /**
     * 根据产品条码查询异常下线信息
     * @param organizeCode 组织代码
     * @param serialNumber  产品条码
     * @return 异常下线信息
     */
    @ApiOperation(value = "根据产品条码查询异常下线信息", notes = "根据产品条码查询异常下线信息")
    MesErrorLineOff getErrorLineOffBySn(String organizeCode, String serialNumber);

    /**
     * 根据产品条码,工单号查询异常下线信息
     * @param organizeCode 组织代码
     * @param serialNumber  产品条码
     * @param workOrderNo  工单号
     * @return 异常下线信息
     */
    @ApiOperation(value = "根据产品条码,工单号查询异常下线信息", notes = "根据产品条码,工单号查询异常下线信息")
    MesErrorLineOff getErrorLineOffBySnAndOrder(String organizeCode, String workOrderNo, String serialNumber);

    /**
     * 根据异常下线ID查询异常下线明细信息集合
     * @param pid 组织代码
     * @return 异常下线明细信息集合
     */
    @ApiOperation(value = "根据异常下线ID查询异常下线明细信息集合", notes = "根据异常下线ID查询异常下线明细信息集合")
    List<MesErrorLineOffDetail> getErrorLineOffDetailListByPid(Long pid);

    /**
     * 根据产品条码,生产工单号查询异常下线明细信息集合
     * @param organizeCode 组织代码
     * @param serialNumber 产品条码
     * @param workOrderNo 生产工单号
     * @return 异常下线明细信息集合
     */
    @ApiOperation(value = "根据产品条码,生产工单号查询异常下线明细信息集合", notes = "根据产品条码,生产工单号查询异常下线明细信息集合")
    List<MesErrorLineOffDetail> getErrorLineOffDetailListBySnAndOrder(String organizeCode, String serialNumber, String workOrderNo);

}
