package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.base.bu;

import cn.estsh.i3plus.ext.mes.pcn.api.base.bu.IBuEquipmentCheckDetailService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesEquipmentCheckDetail;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/2/4 18:05
 **/
@RestController
@Api(tags = "设备点检")
@RequestMapping(CommonConstWords.BASE_URL_MES_PCN + "/bu-ext/equ-check")
public class BuEquipmentCheckDetailController {
    @Autowired
    private IBuEquipmentCheckDetailService mesEquipmentCheckService;

    @GetMapping("query")
    @ApiOperation(value = "条件分页")
    public ResultBean queryPager(MesEquipmentCheckDetail detail, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(this.mesEquipmentCheckService.queryByPager(detail, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("query-cell")
    @ApiOperation(value = "查询工序代码下拉框")
    public ResultBean queryCell(MesEquipmentCheckDetail detail) {
        try {
            return ResultBean.success("查询成功").setResultList(this.mesEquipmentCheckService.queryCell(detail))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 物料点检作业
     *
     * @param detail
     * @return
     */
    @PutMapping("update")
    @ApiOperation(value = "修改")
    public ResultBean update(@RequestBody MesEquipmentCheckDetail detail) {
        try {
            String userName = AuthUtil.getSessionUser().getUserName();
            this.mesEquipmentCheckService.update(detail,userName);
            return ResultBean.success("扫描成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("generate-week")
    @ApiOperation(value = "生成周点检")
    public ResultBean generateWeekCheck(String organizeCode, String workCenterCode) {

        try {
            mesEquipmentCheckService.generateWeekCheck(organizeCode, workCenterCode);
            return ResultBean.success("扫描成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("generate-month")
    @ApiOperation(value = "生成月点检")
    public ResultBean generateMonthCheck(String organizeCode, String workCenterCode) {

        try {
            mesEquipmentCheckService.generateMonthCheck(organizeCode, workCenterCode);
            return ResultBean.success("扫描成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
