package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.bu.oqc;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.bu.IBuQmsOqcManageService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.bu.SxBuOqcDataModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StopWatch;

import java.util.Map;

/**
 * @Description 出货检验 主数据 成品 实现 静态代理
 * @author wangjie
 * @version 1.0
 * @date 2021/1/13 9:28
 **/
@Slf4j
public class BuQmsOqcBySnManageProxy implements IBuQmsOqcManageService {

    private IBuQmsOqcManageService snBusiService;

    public BuQmsOqcBySnManageProxy(IBuQmsOqcManageService snBusiService) {
        this.snBusiService = snBusiService;
    }

    @Override
    public SxBuOqcDataModel exec(Map<String, Object> params) {

        String organizeCode = (String) params.get(MesPcnExtConstWords.ORGANIZE_CODE);

        String serialNumber = (String) params.get(MesPcnExtConstWords.VALUE);

        String dataType = (String) params.get(MesPcnExtConstWords.DATA_TYPE);

        log.info("BU-QMS出货检验主数据 --- 成品出货检验 --- 参数[{}][{}][{}] --- 线程ID[{}]", organizeCode, serialNumber, dataType, Thread.currentThread().getName());

        SxBuOqcDataModel oqcDataModel = snBusiService.exec(params);

        log.info("BU-QMS出货检验主数据 --- 成品出货检验 --- result[{}] --- 线程ID[{}]", JSONObject.toJSONString(oqcDataModel), Thread.currentThread().getName());

        return oqcDataModel;
    }

}

@Slf4j
class BuQmsOqcBySnManageWatchProxy implements IBuQmsOqcManageService {

    private IBuQmsOqcManageService snBusiService;

    public BuQmsOqcBySnManageWatchProxy(IBuQmsOqcManageService snBusiService) {

        this.snBusiService = snBusiService;

    }

    @Override
    public SxBuOqcDataModel exec(Map<String, Object> params) {

        StopWatch stopWatch = new StopWatch();

        stopWatch.start();

        SxBuOqcDataModel oqcDataModel = snBusiService.exec(params);

        stopWatch.stop();

        log.info("BU-QMS出货检验主数据 --- 成品出货检验 SUCCESS --- 耗时[{}] --- 线程ID[{}]", stopWatch.getTotalTimeMillis(), Thread.currentThread().getName());

        return oqcDataModel;
    }

}
