package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.print.method;

import cn.estsh.i3plus.ext.mes.pcn.api.base.ISxWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.ISxPrintTemplateMethodService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesWorkOrderExt;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.ProduceSnExtModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.SxPrintTemplateModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 净水器本体条码打印数据
 * @Author: mingliang.li
 * @CreateDate: 2020/6/22 9:38 AM
 * @Description:
 **/
@Slf4j
@Service
public class SxPrintBodyBarcodeService extends SxPrintDispatchService implements ISxPrintTemplateMethodService {

    @Autowired
    private ISxWorkOrderExtService sxWorkOrderExtService;

    @Override
    public List<Map<String, Object>> exec(SxPrintTemplateModel printModel) {
        List<Map<String, Object>> resultMapList = null;
        List<ProduceSnExtModel> snList = printModel.getSnList();

        List<String> workOrderNoList = snList.stream().filter(o -> null != o).map(ProduceSnExtModel::getWorkOrderNo).collect(Collectors.toList());
        Map<String, String> startTimeMap = new HashMap<>();
        for (String workOrderNo : workOrderNoList) {
            if (StringUtils.isEmpty(workOrderNo)) {
                continue;
            }
            MesWorkOrderExt workOrderExt = sxWorkOrderExtService.getWorkOrderExtByOrderNo(printModel.getOrganizeCode(), workOrderNo);
            if (null == workOrderExt) {
                continue;
            }
            String[] dateArr = workOrderExt.getStartTime().substring(2, 10).split("-");
            startTimeMap.put(workOrderNo, new StringBuilder().append(dateArr[0]).append(dateArr[1]).append(dateArr[2]).toString());
        }

        for (ProduceSnExtModel sn : snList) {
            if (null == sn) {
                continue;
            }
            if (CollectionUtils.isEmpty(resultMapList)) {
                resultMapList = new ArrayList<>();
            }
            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put(MesPcnExtConstWords.SERIAL_NUMBER, getSnWithOutPercent(sn.getSerialNumber()));
            //用于 BarTender 软件 自动换行
            resultMap.put(MesPcnExtConstWords.SERIAL_NUMBER2, getSnWithOutPercent(sn.getSerialNumber()));
            resultMap.put(MesPcnExtConstWords.BAR_CODE, sn.getSerialNumber());
            resultMap.put(MesPcnExtConstWords.ORDER_START_TIME, startTimeMap.get(sn.getWorkOrderNo()));

            resultMapList.add(resultMap);
        }
        return resultMapList;
    }

}
