package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.step;

import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.MesWorkCellJumpSnSeqRecord;
import cn.estsh.i3plus.ext.mes.pcn.pojo.repository.MesWorkCellJumpSnSeqRecordRepository;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.base.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.LocaleUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Map;

/**
 * @Description : 工步业务-本体包装前置扫描跳号码工步
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-07-11
 * @Modify:
 **/
@Slf4j
@Service("sxNoumenonPackagePreScanProdSeqStepService")
public class SxNoumenonPackagePreScanProdSeqStepService extends BaseStepService {

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Autowired
    private MesWorkCellJumpSnSeqRecordRepository workCellJumpSnSeqRecordRepository;

    @Override
    public StepResult guide(StationRequestBean reqBean) {
        StationResultBean resultBean = new StationResultBean();
        String guide = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "请扫描跳号码");
        this.sendMessage(reqBean, resultBean, guide, MesPcnEnumUtil.STATION_BUSI_TYPE.GUIDE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
        return super.guide(reqBean);
    }

    @Override
    public StepResult init(StationRequestBean reqBean) {
        return super.init(reqBean);
    }

    @Override
    public StepResult execute(StationRequestBean reqBean) {
        log.info("SxNoumenonPackagePreScanProdSeqStepService[execute] 参数：{}", reqBean);

        StationResultBean resultBean = new StationResultBean();
        String err;

        if (StringUtils.isEmpty(reqBean.getOrganizeCode())) {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "入参缺少工厂代码");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        if (StringUtils.isEmpty(reqBean.getScanInfo())) {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "入参缺少跳号码");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        if (StringUtils.isEmpty(reqBean.getWorkOrderNo())) {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "入参缺少工单号");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        if (StringUtils.isEmpty(reqBean.getSerialNumber())) {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "入参缺少产品条码");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        Map<String, String> wcpcMap  = fsmCommonService.getCachedWorkCellParam(reqBean);
        if (CollectionUtils.isEmpty(wcpcMap) || (!wcpcMap.containsKey("CHECK_SN_SEQ") || StringUtils.isEmpty(wcpcMap.get("CHECK_SN_SEQ")))) {
            err = String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "工位[%s]未配置跳号码"), reqBean.getWorkCellCode());
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        if (wcpcMap.get("CHECK_SN_SEQ").equals(reqBean.getScanInfo())) {
            err = String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "跳号码验证成功,产品条码[%s]允许跳号执行"), reqBean.getSerialNumber());
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);

            MesWorkCellJumpSnSeqRecord record = new MesWorkCellJumpSnSeqRecord();
            record.setWorkCenterCode(reqBean.getWorkCenterCode());
            record.setWorkCellCode(reqBean.getWorkCellCode());
            record.setWorkOrderNo(reqBean.getWorkOrderNo());
            record.setSerialNumber(reqBean.getSerialNumber());
            record.setLastSerialNumber(StringUtils.isEmpty(reqBean.getDataMap().get("LAST_SN")) ? null : reqBean.getDataMap().get("LAST_SN").toString());
            record.setJumpCode(reqBean.getScanInfo());
            record.setOrganizeCode(reqBean.getOrganizeCode());
            ConvertBean.serviceModelInitialize(record, reqBean.getUserInfo());
            workCellJumpSnSeqRecordRepository.insert(record);

            return StepResult.getSuccessComplete(err);
        } else {
            err = String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "跳号码验证失败,产品条码[%s]不允许跳号执行,请重新扫描产品条码"), reqBean.getSerialNumber());
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            return StepResult.getNonComplete(err);
        }
    }

}
