package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi.step.ep;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IProdBindRecordExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IRouteProcessExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IErrorLineOffService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMissingPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.bean.*;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.ProcessBomModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.repository.MesKeyPartDataRepository;
import cn.estsh.i3plus.ext.mes.pcn.pojo.repository.MesProcessBomRepository;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.ConvertBeanExt;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtEnumUtil;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.IShippingDispatchService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.base.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.LocaleUtil;
import cn.estsh.i3plus.mes.pcn.util.PojoAttrUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.platform.common.util.MesPcnConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesKpsnRule;
import cn.estsh.i3plus.pojo.mes.bean.MesProdBindRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRouteOptParam;
import cn.estsh.i3plus.pojo.mes.bean.MesRouteProcess;
import cn.estsh.i3plus.pojo.mes.model.AttrBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesKpsnRuleRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProdBindRecordRepository;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Description : 工步业务-EP绑定关键件
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-18
 * @Modify:
 **/
@Slf4j
@Service("sxEpBindKeyPartStepService")
public class SxEpBindKeyPartStepService extends BaseStepService {

    @Resource(name = MesPcnExtConstWords.REDIS_RES_PCN)
    private ImppRedis redisMesPcn;

    @Autowired
    private MesKpsnRuleRepository kpsnRuleRepository;

    @Autowired
    private IProduceSnExtService produceSnExtService;

    @Autowired
    private MesProdBindRecordRepository prodBindRecordRepository;

    @Autowired
    private IProdBindRecordExtService prodBindRecordExtService;

    @Autowired
    private MesProcessBomRepository processBomRepository;

    @Autowired
    private IShippingDispatchService shippingDispatchService;

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Autowired
    private IMissingPartService missingPartService;

    @Autowired
    private IErrorLineOffService errorLineOffService;

    @Autowired
    private IRouteProcessExtService routeProcessExtService;

    @Autowired
    private MesKeyPartDataRepository keyPartDataRepository;

    private static final String VALIDATION_CODE = "VALIDATION_CODE";
    private static final String RELOAD_KEY_PART_CODE = "RELOAD_KEY_PART_CODE";
    private static final String EXEC_BIND_KEY = "EXEC_BIND_KEY";
    private static final String MISSING_PART_FLAG = "MISSING_PART_FLAG";
    private static final String FRE_PART_CELL = "FRE_PART_CELL";
    public static final String IS_SAVE_KEY_PART = "IS_SAVE_KEY_PART";
    //新增
    private static final int INSERT = 1;
    //修改
    private static final int UPDATE = 2;
    //修改状态
    private static final int UPDATE_STATUS = 3;

    @Override
    public StepResult init(StationRequestBean reqBean) {
        log.info("SxEpBindKeyPartStepService[init] ws请求参数 {}", reqBean);
        StationResultBean resultBean = new StationResultBean();
        String err;

        MesProduceSnExt produceSnExt;
        if (reqBean != null && !StringUtils.isEmpty(reqBean.getSerialNumber())) {
            produceSnExt = produceSnExtService.getProduceSnExtBySn(reqBean.getOrganizeCode(), reqBean.getSerialNumber());
            if (produceSnExt == null) {
                err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "条码在系统中不存在");
                this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
                return StepResult.getNonComplete(err);
            }
        } else {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "条码在系统中不存在");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            return StepResult.getNonComplete(err);
        }

        String bindKeyPartCacheKey = getCacheKey(reqBean, MesPcnExtConstWords.BIND_KEY_PART);
        List<ProcessBomModel> pbList = (List<ProcessBomModel>) redisMesPcn.getHash(bindKeyPartCacheKey, MesPcnExtConstWords.BIND_KEY_PART);
        if (!CollectionUtils.isEmpty(pbList)) {
            return StepResult.getSuccessComplete();
        }

        pbList = getProcessBomList(reqBean, produceSnExt);

        if (CollectionUtils.isEmpty(pbList)) {
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "没有需要绑定的清单，请维护！");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
        } else {
            List<String> itemPartNoList = new ArrayList<>();
            pbList.forEach(p -> itemPartNoList.add(p.getItemPartNo()));
            List<MesKpsnRule> kpsnRuleList = getKpsnRuleList(reqBean, itemPartNoList);
            for (ProcessBomModel pb : pbList) {
                if (!CollectionUtils.isEmpty(kpsnRuleList)) {
                    for (MesKpsnRule mesKpsnRule : kpsnRuleList) {
                        if (mesKpsnRule.getPartNo().equals(pb.getItemPartNo())) {
                            pb.setLength(mesKpsnRule.getLength());
                            pb.setCheckContent(StringUtils.isEmpty(mesKpsnRule.getFix1()) ? mesKpsnRule.getFix2() : mesKpsnRule.getFix1());
                            break;
                        }
                    }
                }
            }

            Optional<Map<String, MesProdRouteOptParam>> stepParamMap = getStepParams(reqBean);
            String missingPartFlag = null;
            if (stepParamMap.isPresent()) {
                MesProdRouteOptParam param = stepParamMap.get().get(MISSING_PART_FLAG);
                if (null != param && !StringUtils.isEmpty(param.getParamValue())) {
                    missingPartFlag = param.getParamValue();
                }
            }

            checkAndPackPbListIsExistBindedData(pbList, reqBean, produceSnExt, missingPartFlag);

            redisMesPcn.putHash(bindKeyPartCacheKey, MesPcnExtConstWords.BIND_KEY_PART, pbList, MesPcnEnumUtil.EXPIRE_TIME.NEVER.getValue());
            this.refresh(reqBean, pbList);
        }
        return StepResult.getSuccessComplete();
    }

    @Override
    public StepResult execute(StationRequestBean reqBean) {
        log.info("SxEpBindKeyPartStepService[execute] ws请求参数 {}", reqBean);
        StationResultBean resultBean = new StationResultBean();
        String err;

        String bindKeyPartCacheKey = getCacheKey(reqBean, MesPcnExtConstWords.BIND_KEY_PART);

        Map<String, String> wcpcMap = fsmCommonService.getCachedWorkCellParam(reqBean);
        if (!CollectionUtils.isEmpty(wcpcMap) && wcpcMap.containsKey(RELOAD_KEY_PART_CODE) && !StringUtils.isEmpty(wcpcMap.get(RELOAD_KEY_PART_CODE)) &&
                !StringUtils.isEmpty(reqBean.getScanInfo()) && wcpcMap.get(RELOAD_KEY_PART_CODE).equals(reqBean.getScanInfo())) {
            redisMesPcn.deleteKey(bindKeyPartCacheKey);
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "关键件重载码验证成功!即将重载关键件...");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.GUIDE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            StationRequestBean chReq = new StationRequestBean();
            chReq.setClientInfo(shippingDispatchService.getActorClientInfo(reqBean));
            chReq.setInterfaceType(MesPcnConstWords.SHIPPING);
            chReq.setBusiType(MesPcnConstWords.WS_CMD_DO_SCAN);
            chReq.setUserInfo(reqBean.getUserInfo());
            chReq.setOrganizeCode(reqBean.getOrganizeCode());
            chReq.setWorkCellCode(reqBean.getWorkCellCode());
            chReq.setWorkCenterCode(reqBean.getWorkCenterCode());
            chReq.setTriggerAutoFsm(true);
            shippingDispatchService.doSendScanQueueNextExec(chReq);
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "重载关键件成功!");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            foundExThrowNoShowMsg();
        }

        List<ProcessBomModel> pbList = (List<ProcessBomModel>) redisMesPcn.getHash(bindKeyPartCacheKey, MesPcnExtConstWords.BIND_KEY_PART);

        Optional<Map<String, MesProdRouteOptParam>> stepParamMap = getStepParams(reqBean);
        if (stepParamMap.isPresent()) {
            MesProdRouteOptParam param1 = stepParamMap.get().get(VALIDATION_CODE);
            if (null != param1 && !StringUtils.isEmpty(param1.getParamValue()) &&
                    !StringUtils.isEmpty(reqBean.getScanInfo()) && param1.getParamValue().equals(reqBean.getScanInfo())) {
                redisMesPcn.deleteKey(bindKeyPartCacheKey);
                return StepResult.getSuccessComplete();
            }
        }

        if (CollectionUtils.isEmpty(pbList)) {
            this.init(reqBean);
            pbList = (List<ProcessBomModel>) redisMesPcn.getHash(bindKeyPartCacheKey, MesPcnExtConstWords.BIND_KEY_PART);
            if (CollectionUtils.isEmpty(pbList)) {
                redisMesPcn.deleteKey(bindKeyPartCacheKey);
                return StepResult.getSuccessComplete();
            }
        } else {
            this.refresh(reqBean, pbList);
        }

        List<String> itemPartNoList = new ArrayList<>();
        pbList.forEach(p -> itemPartNoList.add(p.getItemPartNo()));
        List<MesKpsnRule> kpsnRuleList = new ArrayList<>();
        for (String itemPartNo : itemPartNoList) {
            List<MesKpsnRule> itemKpsnRuleList = getKpsnRuleList(reqBean, itemPartNo);
            if (CollectionUtils.isEmpty(itemKpsnRuleList)) {
                sendMessage(reqBean, resultBean,
                        String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "物料号[%s]未配置关键件条码校验规则"), itemPartNo),
                        MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
                foundExThrowNoShowMsg();
            }
            kpsnRuleList.addAll(itemKpsnRuleList);
        }

        String missingPartFlag = null;
        if (stepParamMap.isPresent()) {
            MesProdRouteOptParam param = stepParamMap.get().get(MISSING_PART_FLAG);
            if (null != param && !StringUtils.isEmpty(param.getParamValue())) {
                missingPartFlag = param.getParamValue();
            }
        }

        List<ProcessBomModel> finalProcessBomModelList;
        if (!StringUtils.isEmpty(missingPartFlag) && missingPartFlag.equals(CommonEnumUtil.IS_VAILD.VAILD.getValueStr())) {
            finalProcessBomModelList = pbList.stream().filter(o -> (null != o && !o.getIsMissingVal())).collect(Collectors.toList());
        } else {
            finalProcessBomModelList = pbList;
        }
        Optional<ProcessBomModel> unBindOpt = CollectionUtils.isEmpty(finalProcessBomModelList) ? null : finalProcessBomModelList.stream().filter(o -> !o.getIsBindVal()).findFirst();
        if (null == unBindOpt || !unBindOpt.isPresent()) {            //所有的待绑定清单都已绑定完毕
            //this.refresh(reqBean, Arrays.asList());
            redisMesPcn.deleteKey(bindKeyPartCacheKey);
            err = String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "产品条码[%s]:关键件绑定完毕!"), reqBean.getSerialNumber());
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
            return StepResult.getSuccessComplete();
        }

        String scanInfo = reqBean.getScanInfo();
        List<MesKpsnRule> okList = null;
        if (!StringUtils.isEmpty(scanInfo)) {
            for (MesKpsnRule item : kpsnRuleList) {
                if (scanInfo.length() != item.getLengthVal()) {
                    continue;
                }
                if (!scanInfo.substring(item.getFrom1Val() - 1, item.getTo1Val()).equals(item.getFix1())) {
                    continue;
                }
                if (item.getFrom2Val() != 0 && item.getTo2Val() != 0 && !StringUtils.isEmpty(item.getFix2())) {
                    if (!scanInfo.substring(item.getFrom2Val() - 1, item.getTo2Val()).equals(item.getFix2())) {
                        continue;
                    }
                }
                if (CollectionUtils.isEmpty(okList)) {
                    okList = new ArrayList<>();
                }
                okList.add(item);
            }
        }

        if (CollectionUtils.isEmpty(okList)) {
            Object flag = redisMesPcn.getHash(bindKeyPartCacheKey, EXEC_BIND_KEY);
            if (StringUtils.isEmpty(flag) || !(Boolean) flag) {
                err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "请扫描关键件！");
                this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
                redisMesPcn.putHash(bindKeyPartCacheKey, EXEC_BIND_KEY, true, MesPcnEnumUtil.EXPIRE_TIME.NEVER.getValue());
                foundExThrowNoShowMsg();
            } else {
                sendMessage(reqBean, resultBean,
                        String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "扫描的关键件条码没有匹配到规则，条码无效"), pbList.get(0).getPartNo()),
                        MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
                foundExThrowNoShowMsg();
            }
        }

        List<String> okPartNoList = okList.stream().filter(o -> null != o).map(MesKpsnRule::getPartNo).collect(Collectors.toList());

        Optional<ProcessBomModel> pb;
        if (!StringUtils.isEmpty(missingPartFlag) && missingPartFlag.equals(CommonEnumUtil.IS_VAILD.VAILD.getValueStr())) {
            pb = pbList.stream().filter(o -> (null != o && okPartNoList.contains(o.getItemPartNo()) && (StringUtils.isEmpty(o.getIsBind()) || !o.getIsBind())
                    && (StringUtils.isEmpty(o.getIsMissing()) || !o.getIsMissing()))).findFirst();
        } else {
            pb = pbList.stream().filter(o -> (null != o && okPartNoList.contains(o.getItemPartNo()) && (StringUtils.isEmpty(o.getIsBind()) || !o.getIsBind()))).findFirst();
        }
        if (!pb.isPresent()) {
            sendMessage(reqBean, resultBean,
                    LocaleUtil.transferLanguage(reqBean.getUserInfo(), "绑定数量已满足工位BOM中该关键件的总数量"),
                    MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
            foundExThrowNoShowMsg();
        }

        ProcessBomModel processBomModel = pb.get();
        MesKpsnRule matchKpsnRule = okList.stream().filter(o -> (null != o && o.getPartNo().equals(processBomModel.getItemPartNo()))).collect(Collectors.toList()).get(0);

        //是否可以重复投料
        if (processBomModel.getIsRepeatVal() == CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) {
            MesProdBindRecord pbr = prodBindRecordRepository.getByProperty(
                    new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_VALID, MesPcnExtConstWords.IS_DELETED,
                            MesPcnExtConstWords.ACTION_TYPE, MesPcnExtConstWords.KP_SN},
                    new Object[]{reqBean.getOrganizeCode(), CommonEnumUtil.IS_VAILD.VAILD.getValue(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                            MesPcnEnumUtil.MES_ACTION_TYPE.BIND.getValue(), reqBean.getScanInfo()});
            if (pbr != null) {
                sendMessage(reqBean, resultBean,
                        String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "条码[%s]已被绑定使用过，请确认！"), reqBean.getScanInfo()),
                        MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.EXP_TEXT);
                foundExThrowNoShowMsg();
            }
        }

        //默认不回滚(下面不满绑定数量会抛异常)
        MesProdBindRecordExt prodBindRecordExt = insertMesProdBindRecordExt(reqBean, processBomModel, matchKpsnRule);

        processBomModel.setLength(matchKpsnRule.getLength());
        processBomModel.setCheckContent(StringUtils.isEmpty(matchKpsnRule.getFix1()) ? matchKpsnRule.getFix2() : matchKpsnRule.getFix1());
        processBomModel.setBoundQty(StringUtils.isEmpty(processBomModel.getBoundQtyVal()) ? 1.0d : processBomModel.getBoundQtyVal() + 1.0d);
        if (MathOperation.compareTo(processBomModel.getBoundQty(), processBomModel.getQty()) >= 0) {
            processBomModel.setIsBind(true);
            processBomModel.setIsBindName("是");
            processBomModel.setColor(MesPcnExtConstWords.GREEN);
        }

        log.info("SxEpBindKeyPartStepService[execute] 是否还需进行工序bom绑定{}", pb.isPresent());
        MesKeyPartData partAndRfidRecord = getMesKeyPartAndRfidRecord(reqBean);
        if(wcpcMap.containsKey(IS_SAVE_KEY_PART)){
            if(prodBindRecordExt.getIsBindKey() == MesPcnExtEnumUtil.IS_BIND_KEY.YES.getValue() && prodBindRecordExt.getActionType() == MesPcnExtEnumUtil.MES_ACTION_TYPE.BIND.getValue()){

                log.info("SxEpBindKeyPartStepService[execute] 关键件绑定数据:{}",partAndRfidRecord);
                if (pb.isPresent()) {
                    if(partAndRfidRecord == null){
                        partAndRfidRecord =  saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.DOING.getValue(), INSERT);
                    }else{
                        partAndRfidRecord = saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.DOING.getValue(), UPDATE);
                    }
                }else{
                    if(partAndRfidRecord == null){
                        partAndRfidRecord = saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), INSERT);
                    }else{
                        partAndRfidRecord = saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), UPDATE);
                    }
                }
//                if(partAndRfidRecord == null){
//                    saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), INSERT);
//                }else{
//                    saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), UPDATE);
//                }
            }
        }

        redisMesPcn.putHash(bindKeyPartCacheKey, MesPcnExtConstWords.BIND_KEY_PART, pbList, MesPcnEnumUtil.EXPIRE_TIME.NEVER.getValue());
        this.refresh(reqBean, pbList);
        sendMessage(reqBean, resultBean,
                String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "条码：%s 绑定成功！"), reqBean.getScanInfo()),
                MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);

        if (!StringUtils.isEmpty(missingPartFlag) && missingPartFlag.equals(CommonEnumUtil.IS_VAILD.VAILD.getValueStr())) {
            pbList = pbList.stream().filter(o -> (null != o && !o.getIsMissingVal())).collect(Collectors.toList());
        }
        pb = pbList.stream().filter(a -> !a.getIsBindVal()).findFirst();
        log.info("SxEpBindKeyPartStepService[execute] 是否还需进行已绑定绑定{}", pb.isPresent());
        if (pb.isPresent()) {            //所有的待绑定清单都已绑定完毕
            err = LocaleUtil.transferLanguage(reqBean.getUserInfo(), "请继续扫描关键件！");
            this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.GUIDE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);

            //不能return false 如果状态点设置了FAILURE 会直接进入下一个状态点 而不是STOP
            foundExThrowNoShowMsg();
        }

        if(partAndRfidRecord != null){
            saveKeyPartData(reqBean, prodBindRecordExt, partAndRfidRecord, MesPcnExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue(), UPDATE_STATUS);
        }
        //this.refresh(reqBean, Arrays.asList());
        redisMesPcn.deleteKey(bindKeyPartCacheKey);
        err = String.format(LocaleUtil.transferLanguage(reqBean.getUserInfo(), "产品条码[%s]:关键件绑定完毕!"), reqBean.getSerialNumber());
        this.sendMessage(reqBean, resultBean, err, MesPcnEnumUtil.STATION_BUSI_TYPE.MESSAGE, MesPcnEnumUtil.STATION_DATA_TYPE.TEXT);
        return StepResult.getSuccessComplete();
    }

    private MesKeyPartData saveKeyPartData(StationRequestBean reqBean, MesProdBindRecordExt prodBindRecordExt, MesKeyPartData partAndRfidRecord, Integer flag, Integer operationType) {
        log.info("SxEpBindKeyPartStepService[execute] 绑定数据:{}-{}-{}-{}-{}", reqBean, prodBindRecordExt, partAndRfidRecord, flag, operationType);
        if(operationType == UPDATE_STATUS){
            return prodBindRecordExtService.insertKeyPartData(null, reqBean.getUserInfo(), null, partAndRfidRecord, flag, operationType );
        }
        MesKeyPartData keyPartAndRfidRecord = new MesKeyPartData();
        keyPartAndRfidRecord.setProductSn(reqBean.getSerialNumber());
        keyPartAndRfidRecord.setWorkOrderNo(reqBean.getWorkOrderNo());
        keyPartAndRfidRecord.setWorkCenterCode(reqBean.getWorkCenterCode());
        keyPartAndRfidRecord.setPartNo(reqBean.getPartNo());
        keyPartAndRfidRecord.setFlag(flag);
        keyPartAndRfidRecord.setOrganizeCode(reqBean.getOrganizeCode());
        return prodBindRecordExtService.insertKeyPartData(keyPartAndRfidRecord, reqBean.getUserInfo(), prodBindRecordExt, partAndRfidRecord,flag, operationType );
    }

//    private void insertKeyPartAnd(StationRequestBean reqBean, MesProdBindRecordExt prodBindRecordExt) {
//        StringBuffer kpSnStr = new StringBuffer();
//        kpSnStr.append(prodBindRecordExt.getKpSn()).append(MesPcnExtConstWords.COMMA);
//        kpSnStr.deleteCharAt(kpSnStr.length() -1);
//        MesKeyPartData keyPartAndRfidRecord = new MesKeyPartData();
//        keyPartAndRfidRecord.setProductSn(reqBean.getSerialNumber());
//        keyPartAndRfidRecord.setPartNo(reqBean.getPartNo());
//        //打印机读取标志位
//        keyPartAndRfidRecord.setFlag(MesPcnExtEnumUtil.IF_SYNC_STATUS.DOING.getValue());
//        keyPartAndRfidRecord.setKeyData(kpSnStr.toString());
//        ConvertBean.serviceModelInitialize(keyPartAndRfidRecord, reqBean.getUserInfo());
//        keyPartAndRfidRecordRepository.insert(keyPartAndRfidRecord);
//    }

    private MesKeyPartData getMesKeyPartAndRfidRecord(StationRequestBean reqBean) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(reqBean.getWorkOrderNo(), MesPcnExtConstWords.WORK_ORDER_NO, packBean);
        DdlPreparedPack.getStringEqualPack(reqBean.getSerialNumber(), MesPcnExtConstWords.PRODUCT_SN, packBean);
        return keyPartDataRepository.getByProperty(packBean);
    }

    private MesProdBindRecordExt insertMesProdBindRecordExt(StationRequestBean reqBean, ProcessBomModel processBomModel, MesKpsnRule matchKpsnRule) {
        MesProduceSnExt produceSnExt = produceSnExtService.getProduceSnExtBySn(reqBean.getOrganizeCode(), reqBean.getSerialNumber());
        MesProdBindRecordExt pbr = new MesProdBindRecordExt();
        pbr.setWorkOrderNo(produceSnExt.getWorkOrderNo());
        pbr.setSerialNumber(produceSnExt.getSerialNumber());
        pbr.setPartNo(produceSnExt.getPartNo());
        pbr.setWorkCenterCode(reqBean.getWorkCenterCode());
        pbr.setWorkCellCode(reqBean.getWorkCellCode());
        pbr.setItemPartNo(processBomModel.getItemPartNo());
        pbr.setItemPartName(processBomModel.getItemPartName());
        pbr.setKpSn(reqBean.getScanInfo());
        pbr.setKpQty(1.0d);
        pbr.setIsBindKey(processBomModel.getIsBindKey());
        pbr.setActionType(MesPcnEnumUtil.MES_ACTION_TYPE.BIND.getValue());
        pbr.setOrganizeCode(reqBean.getOrganizeCode());
        pbr.setSupplierCode(matchKpsnRule.getSupplierCode());
        pbr.setProcessCode(processBomModel.getProcessCode());
        pbr.setProcessName(processBomModel.getProcessName());
        ConvertBean.serviceModelInitialize(pbr, reqBean.getUserInfo());
        new ConvertBeanExt(pbr).convertBean(pbr);
        prodBindRecordExtService.insertProdBindRecordExtNoRollback(pbr);
        return pbr;
    }

    @Override
    public StepResult jumpStep(StationRequestBean reqBean) {
        String bindKeyPartCacheKey = getCacheKey(reqBean, MesPcnExtConstWords.BIND_KEY_PART);
        redisMesPcn.deleteKey(bindKeyPartCacheKey);
        return super.jumpStep(reqBean);
    }

    private void refresh(StationRequestBean reqBean, List<ProcessBomModel> pbList) {
        List<AttrBean> attrBeanList = new ArrayList<>();
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.ITEM_PART_NO, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "原材料物料号"));
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.ITEM_PART_NAME, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "原材料物料名称"));
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.QTY, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "需求数量"));
//        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.CHECK_CONTENT, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "校验字符"));
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.BOUND_QTY, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "已绑数量"));
        PojoAttrUtil.loadPojoAttrs(attrBeanList, MesPcnExtConstWords.IS_BIND_NAME, LocaleUtil.transferLanguage(reqBean.getUserInfo(), "是否绑定"));

        StationResultBean resultBean = new StationResultBean();
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CONTENT.getValue());
        resultBean.setDataType(MesPcnEnumUtil.STATION_DATA_TYPE.TABLE.getValue());
        resultBean.setResultList(pbList);
        resultBean.setDataAttrList(attrBeanList);
        this.sendMessage(reqBean, resultBean);
    }

    private List<ProcessBomModel> getProcessBomList(StationRequestBean reqBean, MesProduceSnExt produceSnExt) {
        List<ProcessBomModel> pbList = null;
        List<MesProcessBom> processBomList = new ArrayList<>();

        Map<String, String> wcpcMap = fsmCommonService.getCachedWorkCellParam(reqBean);
        if (CollectionUtils.isEmpty(wcpcMap) || !wcpcMap.containsKey(FRE_PART_CELL)) {
            processBomList = getProcessBomListByProcessCodeAndPartNo(reqBean.getOrganizeCode(), reqBean.getProcessCode(), produceSnExt.getPartNo());
            if (CollectionUtils.isEmpty(processBomList)) {
                return pbList;
            }

            for (MesProcessBom processBom : processBomList) {
                if (null == processBom) {
                    continue;
                }
                ProcessBomModel pb = new ProcessBomModel();
                BeanUtils.copyProperties(processBom, pb);
                if (CollectionUtils.isEmpty(pbList)) {
                    pbList = new ArrayList<>();
                }
                pbList.add(pb);
            }
        } else {
            List<MesErrorLineOffDetail> errorLineOffDetailList = errorLineOffService
                    .getErrorLineOffDetailListBySnAndOrder(reqBean.getOrganizeCode(), produceSnExt.getSerialNumber(), produceSnExt.getWorkOrderNo());
            if (!CollectionUtils.isEmpty(errorLineOffDetailList)) {

                List<MesRouteProcess> routeProcessList = routeProcessExtService.getRouteProcessByRoute(reqBean.getOrganizeCode(), reqBean.getRouteCode());
                for (MesRouteProcess routeProcess : routeProcessList) {
                    if (null == routeProcess) {
                        continue;
                    }
                    processBomList.addAll(getProcessBomListByProcessCodeAndPartNo(reqBean.getOrganizeCode(), routeProcess.getProcessCode(), produceSnExt.getPartNo()));
                }

                Map<String, List<MesProcessBom>> bindKeyPartNoMap = CollectionUtils.isEmpty(processBomList) ? null :
                        processBomList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesProcessBom::getItemPartNo));
                errorLineOffDetailList = errorLineOffDetailList.stream()
                        .filter(o -> (null != o && !CollectionUtils.isEmpty(bindKeyPartNoMap) && bindKeyPartNoMap.containsKey(o.getItemPartNo()))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(errorLineOffDetailList)) {
                    for (MesErrorLineOffDetail errorLineOffDetail : errorLineOffDetailList) {
                        if (null == errorLineOffDetail) {
                            continue;
                        }
                        List<MesProcessBom> itemList = bindKeyPartNoMap.get(errorLineOffDetail.getItemPartNo());
                        ProcessBomModel pb = new ProcessBomModel();
                        BeanUtils.copyProperties(itemList.get(0), pb);
                        pb.setQty(itemList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getQty()))).mapToDouble(MesProcessBom::getQty).sum());
                        if (CollectionUtils.isEmpty(pbList)) {
                            pbList = new ArrayList<>();
                        }
                        pbList.add(pb);
                    }
                }
            }
        }
        return pbList;
    }

    private List<MesProcessBom> getProcessBomListByProcessCodeAndPartNo(String organizeCode, String processCode, String partNo) {
        return processBomRepository.findByProperty(
                new String[]{MesPcnExtConstWords.ORGANIZE_CODE, MesPcnExtConstWords.IS_DELETED, MesPcnExtConstWords.IS_VALID,
                        MesPcnExtConstWords.IS_BIND_KEY, MesPcnExtConstWords.PROCESS_CODE, MesPcnExtConstWords.PART_NO},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(), processCode, partNo});
    }

    private List<MesKpsnRule> getKpsnRuleList(StationRequestBean reqBean, List<String> itemPartNoList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
        DdlPreparedPack.getInPackList(itemPartNoList, MesPcnExtConstWords.PART_NO, packBean);
        return kpsnRuleRepository.findByHqlWhere(packBean);
    }

    private List<MesKpsnRule> getKpsnRuleList(StationRequestBean reqBean, String itemPartNo) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(itemPartNo, MesPcnExtConstWords.PART_NO, packBean);
        return kpsnRuleRepository.findByHqlWhere(packBean);
    }

    private void checkAndPackPbListIsExistBindedData(List<ProcessBomModel> pbList, StationRequestBean reqBean, MesProduceSnExt produceSnExt, String missingPartFlag) {
        List<MesProdBindRecord> prodBindRecordList = prodBindRecordExtService
                .getProdBindRecordList(reqBean.getOrganizeCode(), produceSnExt.getWorkOrderNo(), produceSnExt.getSerialNumber(), reqBean.getWorkCellCode());
        List<String> missingPartNoList = null;
        if (!StringUtils.isEmpty(missingPartFlag)) {
            if (missingPartFlag.equals(CommonEnumUtil.IS_VAILD.VAILD.getValueStr())) {
                //正常绑定关键件工位突出显示缺件的数据（在下面的逻辑中体现）
                List<MesMissingPart> missingPartList = missingPartService.getMissingPartByWorkOrderNo(reqBean.getOrganizeCode(), produceSnExt.getWorkOrderNo());
                missingPartNoList = CollectionUtils.isEmpty(missingPartList) ? new ArrayList<>() :
                        missingPartList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getItemPartNo()))).map(MesMissingPart::getItemPartNo).collect(Collectors.toList());
            } else {
                //缺件：pbList数据就是已缺件数据
                missingPartNoList = pbList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getItemPartNo()))).map(ProcessBomModel::getItemPartNo).collect(Collectors.toList());
            }
        }

        Map<String, List<MesProdBindRecord>> bindedMap = CollectionUtils.isEmpty(prodBindRecordList) ? null : prodBindRecordList.stream()
                .filter(o -> (null != o && !StringUtils.isEmpty(o.getItemPartNo()))).collect(Collectors.groupingBy(MesProdBindRecord::getItemPartNo));
        for (ProcessBomModel pb : pbList) {
            if (null == pb) {
                continue;
            }
            if (!StringUtils.isEmpty(missingPartFlag) && missingPartNoList.contains(pb.getItemPartNo())) {
                pb.setIsMissing(true);
                pb.setIsBindName("缺件");
                pb.setColor(MesPcnExtConstWords.YELLOW);
            }
            if (!CollectionUtils.isEmpty(bindedMap)) {
                if (!bindedMap.containsKey(pb.getItemPartNo())) {
                    continue;
                }
                pb.setBoundQty(new Double(bindedMap.get(pb.getItemPartNo()).size()));
                if (MathOperation.compareTo(pb.getBoundQty(), pb.getQty()) >= 0) {
                    pb.setIsBind(true);
                    pb.setIsBindName("是");
                    pb.setColor(MesPcnExtConstWords.GREEN);
                }
            }
        }
    }

}

